/*
 * QEMU4V tracer
 *
 *  Copyright (c) 2015-2018 Sergey Smolov
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef Q4V_TRACER_H_
#define Q4V_TRACER_H_

#include <string.h>
#include <stdbool.h>
#include <stdint.h>

/*
 * Functions that set\check tracing mode.
 */
void enable_q4v_trace(void);
bool q4v_trace_enabled(void);

/*
 * Functions that set\check Page Table Entry tracing mode.
 */
void enable_q4v_pte(void);
bool q4v_pte_enabled(void);

/*
 * Time stamp value functions.
 */
int q4v_tstamp(void);
void set_q4v_tstamp(int tstamp);

void q4v_trace_write_regname_prefix(const char *regname);

/*
 * Function that stores prefix for register write event in tracer data.
 */
void log_write_prefix(const int reg);

void q4v_trace_value(const uint64_t value);

/**
 * Function for register write tracing.
 */
void q4v_trace_reg_write(const char *regname, const uint64_t value);

/**
 * Function for memory access (read\write) tracing.
 */
void q4v_trace_mem_access(const char *rw,
    const int size, const uint64_t mem_addr, const uint64_t data);

/* Utility functions. */
char *trimwhitespace(char *str);
char* insn_strcat(char *insn_str, const char *str);

/* The assembly instruction descriptor. */
struct insninfo {

    /* Instruction address. */
    uint64_t insn_addr;

    /* Instruction string representation. */
    char insn[256];

    /* Pointer to the next instruction in this translation block. */
    struct insninfo *next;
};

/* Sequence of insns for current translation block (filled in disas.c). */
extern struct insninfo *insninfos;

#endif /* Q4V_TRACER_H_ */
